@@ -25,8 +25,8 @@
  */
 
 #include "dunbash.h"
+#include "combat.h"
 #include <stdio.h>
-#include <stdlib.h>
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -66,10 +66,22 @@ static void rebuild_mapobjs(void)
 	}
 }
 
+unsigned int convert_range(int dy, int dx)
+{
+	int ady, adx;
+	ady = dy > 0 ? dy : -dy;
+	adx = dx > 0 ? dx : -dx;
+	if (ady > adx)
+	{
+		return ady;
+	}
+	return adx;
+}
+
 void save_game(void)
 {
 	FILE *fp;
-	fp = fopen("7drl.sav", "wb");
+	fp = fopen("dunbash.sav", "wb");
 	/* Write out the snapshot we took of the RNG before generating the
 	 * current level. */
 	fwrite(saved_state, sizeof saved_state, 1, fp);
@@ -90,11 +102,13 @@ void save_game(void)
 	fwrite(&depth, 1, sizeof depth, fp);
 	/* Write out the player. */
 	fwrite(&u, 1, sizeof u, fp);
+	/* Write out the tick number */
+	fwrite(&game_tick, 1, sizeof game_tick, fp);
 	/* Clean up */
 	fflush(fp);
 	fclose(fp);
 	/* Compress! */
-	system("gzip 7drl.sav");
+	system("gzip dunbash.sav");
 	game_finished = 1;
 	return;
 }
@@ -102,8 +116,8 @@ void save_game(void)
 void load_game(void)
 {
 	FILE *fp;
-	system("gunzip 7drl.sav");
-	fp = fopen("7drl.sav", "rb");
+	system("gunzip dunbash.sav");
+	fp = fopen("dunbash.sav", "rb");
 	fread(rng_state, sizeof rng_state, 1, fp);
 	room_reset();
 	build_level();
@@ -115,8 +129,9 @@ void load_game(void)
 	rebuild_mapmons();
 	fread(&depth, 1, sizeof depth, fp);
 	fread(&u, 1, sizeof u, fp);
+	fread(&game_tick, 1, sizeof game_tick, fp);
 	fclose(fp);
-	unlink("7drl.sav");
+	unlink("dunbash.sav");
 	status_updated = 1;
 	map_updated = 1;
 	print_msg("Game loaded.\n");
@@ -221,23 +236,13 @@ int do_command(enum game_cmd cmd)
 
 	case WIELD_WEAPON:
 		j = 0;
-		if (u.weapon >= 0)
+		i = inv_select(POCLASS_WEAPON, "wield", 1);
+		if (i == -2)
 		{
-			print_msg("You are already wielding a weapon.\n");
-			j = getyn("Do you want to change weapons?\n");
-			if (j == 1)
-			{
-				print_msg("Weapon unwielded. To use your bare hands, select\nno weapon.\n");
-				u.weapon = 0;
-			}
-			else
-			{
-				print_msg("Cancelled.\n");
-				return 0;
-			}
+			u.weapon = -1;
+			print_msg("Weapon unwielded.\n");
 		}
-		i = inv_select(POCLASS_WEAPON, "wield");
-		if (i >= 0)
+		else if (i >= 0)
 		{
 			u.weapon = u.inventory[i];
 			j = 1;
@@ -253,14 +258,15 @@ int do_command(enum game_cmd cmd)
 			print_msg("You are already wearing armour.\n");
 			return 0;
 		}
-		i = inv_select(POCLASS_ARMOUR, "wear");
+		i = inv_select(POCLASS_ARMOUR, "wear", 0);
 		if (i >= 0)
 		{
 			u.armour = u.inventory[i];
-			/* Fix (1.1): Magic armour now auto-IDs when worn. */
 			permobjs[objects[u.armour].obj_id].known = 1;
 			recalc_defence();
-			print_msg("You change your armour.\n");
+			print_msg("Wearing ");
+			print_obj_name(u.armour);
+			print_msg(".\n");
 			return 1;
 		}
 		return 0;
@@ -275,7 +281,7 @@ int do_command(enum game_cmd cmd)
 		return 0;
 
 	case READ_SCROLL:
-		i = inv_select(POCLASS_SCROLL, "read");
+		i = inv_select(POCLASS_SCROLL, "read", 0);
 		if (i >= 0)
 		{
 			j = read_scroll(u.inventory[i]);
@@ -288,15 +294,35 @@ int do_command(enum game_cmd cmd)
 		return 0;
 
 	case GO_DOWN_STAIRS:
-		leave_level();
-		make_new_level();
+		if (terrain[u.y][u.x] == STAIRS)
+		{
+			leave_level();
+			make_new_level();
+		}
+		else
+		{
+			print_msg("There are no stairs here.\n");
+		}
 		return 0;
 
 	case STAND_STILL:
 		return 1;
 
+	case EAT_FOOD:
+		i = inv_select(POCLASS_FOOD, "eat", 0);
+		if (i >= 0)
+		{
+			j = eat_food(u.inventory[i]);
+			if (j == -1)
+			{
+				u.inventory[i] = -1;
+			}
+
+		}
+		break;
+
 	case QUAFF_POTION:
-		i = inv_select(POCLASS_POTION, "quaff");
+		i = inv_select(POCLASS_POTION, "quaff", 0);
 		if (i >= 0)
 		{
 			j = quaff_potion(u.inventory[i]);
@@ -353,7 +379,7 @@ int do_command(enum game_cmd cmd)
 			print_msg("You are already wearing a ring.\n");
 			return 0;
 		}
-		i = inv_select(POCLASS_RING, "put on");
+		i = inv_select(POCLASS_RING, "put on", 0);
 		if (i >= 0)
 		{
 			u.ring = u.inventory[i];
@@ -364,7 +390,12 @@ int do_command(enum game_cmd cmd)
 		}
 		return 0;
 	case DROP_ITEM:
-		i = inv_select(POCLASS_NONE, "drop");
+		if (mapobject[u.y][u.x] != -1)
+		{
+			print_msg("There is already an item here.\n");
+			return 0;
+		}
+		i = inv_select(POCLASS_NONE, "drop", 0);
 		if (i >= 0)
 		{
 			if ((u.inventory[i] != -1) &&
@@ -387,7 +418,7 @@ int do_command(enum game_cmd cmd)
 		save_game();
 		return 0;
 	case QUIT:
-		j = getyn("Really quit?\n");
+		j = getYN("Really quit?\n");
 		if (j > 0)
 		{
 			game_finished = 1;
@@ -414,16 +445,7 @@ void main_loop(void)
 	print_msg("Press '?' for help.\n");
 	while (!game_finished)
 	{
-		if (!(game_tick % 20) && (u.hpcur != u.hpmax))
-		{
-			heal_u(1, 0);
-		}
-		if (((game_tick % 20) == 10) && (objects[u.ring].obj_id == PO_RING_REGEN) && (u.hpcur != u.hpmax))
-		{
-			print_msg("Your ring pulses soothingly.\n");
-			heal_u(one_die(3), 0);
-			permobjs[PO_RING_REGEN].known = 1;
-		}
+		update_player();
 		switch (game_tick % 4)
 		{
 		case 0:
@@ -438,7 +460,7 @@ void main_loop(void)
 			break;
 		}
 		/* Player is always speed 1, for now. */
-		if (action_speed <= (u.armour == -1) ? 1 : ((objects[u.armour].obj_id == PO_ROBE_SWIFTNESS) ? 2 : 1))
+		if (action_speed <= u.speed)
 		{
 			i = 0;
 			while (!i)
@@ -461,6 +483,7 @@ void main_loop(void)
 			{
 				print_msg("Your ring pulses uncleanly.\n");
 				damage_u(1, DEATH_KILLED, "a ring of doom");
+				display_update();
 				permobjs[PO_RING_DOOM].known = 1;
 			}
 			else if (objects[u.ring].obj_id == PO_RING_TELEPORT)
@@ -480,13 +503,20 @@ void main_loop(void)
 				/* Unused monster */
 				continue;
 			}
-			/* Trolls now regenerate */
-			if ((monsters[i].mon_id == PM_TROLL) &&
-			    !(game_tick % 10) &&
-			    (monsters[i].hpcur < monsters[i].hpmax))
+			/* Monsters now regenerate */
+			if (monsters[i].hpcur < monsters[i].hpmax)
 			{
-				print_msg("The troll regenerates.\n");
-				heal_mon(i, one_die(3) + 3);
+				if ((monsters[i].mon_id == PM_TROLL) &&
+				    !(game_tick % 10))
+				{
+					print_msg("The troll regenerates.\n");
+					heal_mon(i, one_die(3) + 3);
+				}
+				else if ((monsters[i].mon_id != PM_TROLL) &&
+				    !(game_tick % 20))
+				{
+					heal_mon(i, 1);
+				}
 			}
 			if (action_speed <= permons[monsters[i].mon_id].speed)
 			{
@@ -511,7 +541,7 @@ int main(void)
 	int i;
 	display_init();
 	/* Do we have a saved game? */
-	i = stat("7drl.sav.gz", &s);
+	i = stat("dunbash.sav.gz", &s);
 	if (!i)
 	{
 		/* Yes! */
