@@ -88,8 +88,39 @@ int read_scroll(int obj)
 		}
 		break;
 	case PO_SCR_MONSTERS:
-		print_msg("Monsters appear!\n");
-		summoning(u.y, u.x, one_die(3) + 1);
+		i = summoning(u.y, u.x, one_die(3) + 1);
+		if (i > 0)
+		{
+			print_msg("Monsters appear!\n");
+		}
+		else
+		{
+			print_msg("You hear a snarl of frustration.\n");
+		}
+		break;
+	case PO_SCR_PROTECTION:
+		print_msg("You feel like something is helping you.\n");
+		if (!u.protection)
+		{
+			/* Do not prolong existing protection, only grant
+			 * protection to the unprotected. */
+			u.protection = 100;
+		}
+		if (u.withering)
+		{
+			print_msg("Your limbs straighten.\n");
+			u.withering = 0;
+		}
+		if (u.armourmelt)
+		{
+			print_msg("Your armour regains its strength.\n");
+			u.armourmelt = 0;
+		}
+		if (u.leadfoot)
+		{
+			print_msg("You shed your feet of lead.\n");
+			u.leadfoot = 0;
+		}
 		break;
 	default:
 		print_msg("Impossible: reading non-scroll\n");
@@ -105,6 +136,28 @@ int read_scroll(int obj)
 	return 0;
 }
 
+static int consume_obj(int obj)
+{
+	objects[obj].quan--;
+	if (objects[obj].quan == 0)
+	{
+		objects[obj].used = 0;
+		return 1;
+	}
+	return 0;
+}
+
+int eat_food(int obj)
+{
+	struct obj *optr = objects + obj;
+	if (optr->obj_id != PO_FOOD_RATION)
+	{
+		return -1;
+	}
+	u.food += 2000;
+	return consume_obj(obj);
+}
+
 int quaff_potion(int obj)
 {
 	struct obj *optr = objects + obj;
@@ -117,6 +170,7 @@ int quaff_potion(int obj)
 		break;
 	case PO_POT_AGILITY:
 		u.agility++;
+		recalc_defence();
 		status_updated = 1;
 		print_msg("You feel more agile!\n");
 		break;
@@ -128,9 +182,12 @@ int quaff_potion(int obj)
 	case PO_POT_POISON:
 		print_msg("This is poison!\n");
 		damage_u(dice(3, 12), DEATH_KILLED, "drinking poison");
+		display_update();
 		break;
 	case PO_POT_HEAL:
-		heal_u(dice(3, 12), 0);
+		/* Heal player; if hit points brought to max, gain one
+		 * hit point. */
+		heal_u(dice(3, 12), 1);
 		break;
 	default:
 		print_msg("Impossible: quaffing non-potion\n");
@@ -280,6 +337,14 @@ int get_random_pobj(void)
 			po_idx = -1;
 			continue;
 		}
+		/* v1.3: Do not permit generation of particularly powerful
+		 * items (runeswords, mage armour, etc.) at shallow depths.
+		 * (game balance fix) */
+		if (depth < permobjs[po_idx].depth)
+		{
+			po_idx = -1;
+			continue;
+		}
 		break;
 	}
 	return po_idx;
@@ -340,9 +405,13 @@ void print_obj_name(int obj)
 		{
 			print_msg("%d %s", optr->quan, poptr->plural);
 		}
+		else if (po_is_stackable(optr->obj_id))
+		{
+			print_msg("1 %s", poptr->name);
+		}
 		else
 		{
-			print_msg("a %s", poptr->name);
+			print_msg("a%s %s", is_vowel(poptr->name[0]) ? "n" : "", poptr->name);
 		}
 	}
 	else
@@ -352,11 +421,15 @@ void print_obj_name(int obj)
 		case POCLASS_NONE:
 			print_msg("a non-thing (%d)", optr->obj_id);
 			break;
+		case POCLASS_FOOD:
+			print_msg("a mysterious food (%d)", optr->obj_id);
+			break;
 		case POCLASS_WEAPON:
 			print_msg("a mysterious weapon (%d)", optr->obj_id);
 			break;
 		case POCLASS_ARMOUR:
-			if ((optr->obj_id == PO_ROBE_SHADOWS) ||
+			if ((optr->obj_id == PO_ROBE) ||
+			    (optr->obj_id == PO_ROBE_SHADOWS) ||
 			    (optr->obj_id == PO_ROBE_SWIFTNESS))
 			{
 				print_msg("a robe");
@@ -387,7 +460,7 @@ void print_obj_name(int obj)
 			}
 			break;
 		case POCLASS_RING:
-			print_msg("1 %s ring", ring_colours[poptr->power]);
+			print_msg("a%s %s ring", is_vowel(ring_colours[poptr->power][0]) ? "n" : "", ring_colours[poptr->power]);
 			break;
 		}
 	}
@@ -428,6 +501,7 @@ int po_is_stackable(int po)
 		return 0;
 	case POCLASS_POTION:
 	case POCLASS_SCROLL:
+	case POCLASS_FOOD:
 		return 1;
 	}
 }
